"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Argument = exports.ArgumentError = void 0;
// builtin
const node_process_1 = require("node:process");
// external
const errlop_1 = __importDefault(require("errlop"));
/** Trim the initial indentation for the entire string. */
function trimIndentation(string) {
    const lines = string.split('\n');
    const firstLine = lines[0] || lines[1]; // in case it starts on the second line
    const match = firstLine.match(/^\s+/);
    if (match) {
        const indentation = match[0];
        return lines.map((line) => line.replace(indentation, '')).join('\n');
    }
    else {
        return string;
    }
}
/** An {@link Errlop} instance with defaults configured for argument errors. */
class ArgumentError extends errlop_1.default {
    exitCode = 22;
    code = 'EINVAL';
    orphanStack = ''; // we don't care for the stack for this error
}
exports.ArgumentError = ArgumentError;
/** Parse a command line argument/flag */
class Argument {
    /** The entire argument value */
    arg;
    /**
     * If an argument, this is an empty string.
     * If a flag, this is the name of the flag.
     * If --, this is --
     */
    key;
    /**
     * If an argument, this is the argument.
     * If a flag, this is the value of the flag if present.
     * If --, this is null
     */
    value = null;
    /** If we are a flag */
    flag = false;
    /** If --no- prefix was used, this becomes true. */
    inverted = false;
    /** Values that will be considered as true when parsing as a boolean. */
    truthy = ['true', '1', 'yes', 'y', 'on'];
    /** Values that will be considered as false when parsing as a boolean. */
    falsey = ['false', '0', 'no', 'n', 'off'];
    /** Parse our argument, extracting key, value, and inverted. */
    constructor(arg) {
        this.arg = arg;
        let key, value = null;
        if (arg === '--') {
            key = '--';
        }
        else if (arg.startsWith('--')) {
            this.flag = true;
            const index = arg.indexOf('=');
            if (index === -1) {
                key = arg.substring(2);
            }
            else {
                key = arg.substring(2, index);
                value = arg.substring(index + 1);
            }
            if (key.startsWith('no-')) {
                key = key.substring(3);
                this.inverted = true;
            }
        }
        else {
            key = '';
            value = arg;
        }
        this.key = key;
        this.value = value;
    }
    /** Throw an {@link ArgumentError} with a custom message */
    static error(error) {
        throw new ArgumentError(error);
    }
    /** Throw an {@link ArgumentError} with that will output our help message  */
    static help(help) {
        throw new ArgumentError(help);
    }
    /**
     * Return a catcher that will exit the process with the help message, the error (if any), and the appropriate exit code.
     * @example `.catch(Argument.catch(help))`
     */
    static catch(help) {
        return (error) => this.exit(help, error);
    }
    /** Exit the process with the help message, the error (if any), and the appropriate exit code. */
    static exit(help, error) {
        const string = error?.toString();
        if (error?.exitCode === 22 || !string) {
            console.error(trimIndentation(help).trim());
            if (string && error.message !== help) {
                console.error('\n' + string);
                (0, node_process_1.exit)(22);
            }
            else
                (0, node_process_1.exit)(0);
        }
        else {
            console.error(string);
            (0, node_process_1.exit)(error.exitCode ?? 1);
        }
    }
    /** Throw an {@link ArgumentError} with a custom message */
    error(error) {
        throw Argument.error(error);
    }
    /** Throw an {@link ArgumentError} that this argument is unknown */
    unknown() {
        throw this.error(this.flag ? `Unknown flag: ${this.arg}` : `Unknown argument ${this.arg}`);
    }
    /** Parse the defaults for the argument. */
    fallback(error, { enabled, disabled, enabledEmpty, disabledEmpty } = {}) {
        if (this.value == null) {
            if (this.inverted) {
                if (disabled == null)
                    throw this.error(error);
                return disabled;
            }
            else {
                if (enabled == null)
                    throw this.error(error);
                return enabled;
            }
        }
        else {
            if (this.value === '') {
                if (this.inverted) {
                    if (enabledEmpty == null)
                        throw this.error(error);
                    return enabledEmpty;
                }
                else {
                    if (disabledEmpty == null)
                        throw this.error(error);
                    return disabledEmpty;
                }
            }
            return this.value;
        }
    }
    /** Parse our flag as a string. */
    string(opts = {}) {
        const error = `Argument ${this.arg} must have a string value, e.g. ` +
            (this.flag ? `--${this.key}=string` : '<string>');
        return this.fallback(error, opts);
    }
    /** Parse our argument/flag as a number */
    number(opts = {}) {
        const error = `Argument ${this.arg} must have a number value, e.g. ` +
            (this.flag ? `--${this.key}=123` : '123');
        const value = this.fallback(error, opts);
        const number = value == null || value === '' ? NaN : Number(value);
        if (isNaN(number))
            throw this.error(error);
        return number;
    }
    /** Parse our argument/flag as a boolean */
    boolean() {
        if (this.value == null || this.value === '') {
            return !this.inverted;
        }
        else if (this.truthy.includes(this.value)) {
            if (this.inverted)
                return false;
            return true;
        }
        else if (this.falsey.includes(this.value)) {
            if (this.inverted)
                return true;
            return false;
        }
        else {
            const error = `Argument ${this.arg} must have a boolean value, e.g. ` +
                (this.flag
                    ? `--${this.key} or --no-${this.key} or --${this.key}=yes`
                    : 'yes');
            throw this.error(error);
        }
    }
}
exports.Argument = Argument;
exports.default = Argument;
