import Errlop from 'errlop';
/** Customise the fallbacks. */
export interface FallbackOptions<T> {
    /** when [--key] use this value, otherwise throw */
    enabled?: T | null;
    /** when [--no-key] use this value, otherwise throw */
    disabled?: T | null;
    /** when [--key=] use this value, otherwise throw */
    enabledEmpty?: T | null;
    /** when [--no-key=] use this value, otherwise throw */
    disabledEmpty?: T | null;
}
/** An {@link Errlop} instance with defaults configured for argument errors. */
export declare class ArgumentError extends Errlop {
    readonly exitCode: number;
    readonly code: string | number;
    readonly orphanStack: string;
}
/** Parse a command line argument/flag */
export declare class Argument {
    /** The entire argument value */
    readonly arg: string;
    /**
     * If an argument, this is an empty string.
     * If a flag, this is the name of the flag.
     * If --, this is --
     */
    readonly key: string;
    /**
     * If an argument, this is the argument.
     * If a flag, this is the value of the flag if present.
     * If --, this is null
     */
    readonly value: string | null;
    /** If we are a flag */
    readonly flag: boolean;
    /** If --no- prefix was used, this becomes true. */
    readonly inverted: boolean;
    /** Values that will be considered as true when parsing as a boolean. */
    readonly truthy: string[];
    /** Values that will be considered as false when parsing as a boolean. */
    readonly falsey: string[];
    /** Parse our argument, extracting key, value, and inverted. */
    constructor(arg: string);
    /** Throw an {@link ArgumentError} with a custom message */
    static error(error: any): never;
    /** Throw an {@link ArgumentError} with that will output our help message  */
    static help(help: string): never;
    /**
     * Return a catcher that will exit the process with the help message, the error (if any), and the appropriate exit code.
     * @example `.catch(Argument.catch(help))`
     */
    static catch(help: string): (error: any) => never;
    /** Exit the process with the help message, the error (if any), and the appropriate exit code. */
    static exit(help: string, error?: any): never;
    /** Throw an {@link ArgumentError} with a custom message */
    error(error: any): never;
    /** Throw an {@link ArgumentError} that this argument is unknown */
    unknown(): never;
    /** Parse the defaults for the argument. */
    protected fallback<T>(error: string, { enabled, disabled, enabledEmpty, disabledEmpty }?: FallbackOptions<T>): any;
    /** Parse our flag as a string. */
    string<T>(opts?: FallbackOptions<T>): T;
    /** Parse our argument/flag as a number */
    number<T>(opts?: FallbackOptions<T>): T;
    /** Parse our argument/flag as a boolean */
    boolean(): boolean;
}
export default Argument;
//# sourceMappingURL=index.d.ts.map